// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';
import 'dart:isolate';

import 'package:wyatt_cli_toolbox/src/sequences/sequences.dart';
import 'package:wyatt_cli_toolbox/src/spinners/indicator.dart';
import 'package:wyatt_cli_toolbox/src/spinners/spinner_type.dart';


class Spinner {
  /// Active isolate object. Is created when [start] is called.
  static Isolate? _spinnerIsolate;

  /// Run the animation of active [_spinnerIsolate].
  static Future<void> _animate(Indicator indicator) async {
    indicator.animate();
  }

  /// Starts an animation on a new line.
  static Future<Isolate> start(SpinnerType spinner, {String? text}) async {
    return custom(spinner.indicator.copyWith(label: text));
  }

  /// Starts a custom animation on a new line.
  static Future<Isolate> custom(Indicator indicator) async {
    Sequences.hideCursor();
    return _spinnerIsolate ??
        (_spinnerIsolate = await Isolate.spawn(
          _animate,
          indicator,
        ));
  }

  /// Stops current spinner animation by killing the isolate instance,
  /// and clears current line.
  static void stop() {
    if (_spinnerIsolate != null) {
      _spinnerIsolate!.kill(priority: Isolate.immediate);
    }
    _spinnerIsolate = null;
    Sequences.clearEntireLine();
    Sequences.moveHorizontal(1);
    Sequences.unhideCursor();
  }

  /// Stop and restart a new indicator on same line.
  static void update(Indicator indicator) {
    stop();
    custom(indicator);
  }
}
